@@ -70,6 +70,10 @@ public class HistogramLogProcessor extends Thread {
         public boolean listTags = false;
         public boolean allTags = false;
 
+        public boolean movingWindow = false;
+        public double movingWindowPercentileToReport = 99.0;
+        public long movingWindowLengthInMsec = 60000; // 1 minute
+
         public int percentilesOutputTicksPerHalf = 5;
         public Double outputValueUnitRatio = 1000000.0; // default to msec units for output.
 
@@ -92,6 +96,12 @@ public class HistogramLogProcessor extends Thread {
                         inputFileName = args[++i];
                     } else if (args[i].equals("-tag")) {
                         tag = args[++i];
+                    } else if (args[i].equals("-mwp")) {
+                        movingWindowPercentileToReport = Double.parseDouble(args[++i]);
+                        movingWindow = true;
+                    } else if (args[i].equals("-mwpl")) {
+                        movingWindowLengthInMsec = Long.parseLong(args[++i]);
+                        movingWindow = true;
                     } else if (args[i].equals("-start")) {
                         rangeStartTimeSec = Double.parseDouble(args[++i]);
                     } else if (args[i].equals("-end")) {
@@ -201,9 +211,16 @@ public class HistogramLogProcessor extends Thread {
     @Override
     public void run() {
         PrintStream timeIntervalLog = null;
+        PrintStream movingWindowLog = null;
         PrintStream histogramPercentileLog = System.out;
         Double firstStartTime = 0.0;
         boolean timeIntervalLogLegendWritten = false;
+        boolean movingWindowLogLegendWritten = false;
+
+//        EncodableHistogram[] movingWindow = new EncodableHistogram[config.movingWindowIntervalCount];
+        EncodableHistogram movingWindowSumHistogram = null;
+        Queue<EncodableHistogram> movingWindowQueue = new LinkedList<EncodableHistogram>();
+        int movingWindowIndex = 0;
 
         if (config.listTags) {
             Set<String> tags = new TreeSet<String>();
@@ -229,10 +246,13 @@ public class HistogramLogProcessor extends Thread {
         }
 
         final String logFormat;
+        final String movingWindowLogFormat;
         if (config.logFormatCsv) {
             logFormat = "%.3f,%d,%.3f,%.3f,%.3f,%d,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f\n";
+            movingWindowLogFormat = "%.3f,%d,%.3f,%.3f\n";
         } else {
             logFormat = "%4.3f: I:%d ( %7.3f %7.3f %7.3f ) T:%d ( %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f )\n";
+            movingWindowLogFormat = "%4.3f: I:%d P:%7.3f M:%7.3f\n";
         }
 
         try {
@@ -250,6 +270,16 @@ public class HistogramLogProcessor extends Thread {
                 } catch (FileNotFoundException ex) {
                     System.err.println("Failed to open percentiles histogram output file " + hgrmOutputFileName);
                 }
+                if (config.movingWindow) {
+                    String movingWindowOutputFileName = config.outputFileName + ".mwp";
+                    try {
+                        movingWindowLog = new PrintStream(new FileOutputStream(movingWindowOutputFileName), false);
+                        outputTimeRange(movingWindowLog, "Moving window log for " +
+                                config.movingWindowPercentileToReport + " percentile");
+                    } catch (FileNotFoundException ex) {
+                        System.err.println("Failed to open moving window output file " + movingWindowOutputFileName);
+                    }
+                }
             }
 
             EncodableHistogram intervalHistogram = getIntervalHistogram(config.tag);
@@ -263,14 +293,18 @@ public class HistogramLogProcessor extends Thread {
                     accumulatedDoubleHistogram = ((DoubleHistogram) intervalHistogram).copy();
                     accumulatedDoubleHistogram.reset();
                     accumulatedDoubleHistogram.setAutoResize(true);
+                    movingWindowSumHistogram = new DoubleHistogram(3);
                 } else {
                     accumulatedRegularHistogram = ((Histogram) intervalHistogram).copy();
                     accumulatedRegularHistogram.reset();
                     accumulatedRegularHistogram.setAutoResize(true);
+                    movingWindowSumHistogram = new Histogram(3);
                 }
             }
 
             while (intervalHistogram != null) {
+
+                // handle accumulated histogram:
                 if (intervalHistogram instanceof DoubleHistogram) {
                     if (accumulatedDoubleHistogram == null) {
                         throw new IllegalStateException("Encountered a DoubleHistogram line in a log of Histograms.");
@@ -283,6 +317,34 @@ public class HistogramLogProcessor extends Thread {
                     accumulatedRegularHistogram.add((Histogram) intervalHistogram);
                 }
 
+                long windowCutOffTimeStamp = intervalHistogram.getEndTimeStamp() - config.movingWindowLengthInMsec;
+                // handle moving window:
+                if (config.movingWindow) {
+
+                    // Add the current interval histogram to the moving window sums:
+                    if (movingWindowSumHistogram instanceof DoubleHistogram) {
+                        ((DoubleHistogram) movingWindowSumHistogram).add((DoubleHistogram) intervalHistogram);
+                    } else {
+                        ((Histogram) movingWindowSumHistogram).add((Histogram) intervalHistogram);
+                    }
+                    // Remove previous, now-out-of-window interval histograms from moving window:
+                    while ((movingWindowQueue.peek() != null) &&
+                            (movingWindowQueue.peek().getEndTimeStamp() <= windowCutOffTimeStamp)) {
+                        EncodableHistogram prevHist = movingWindowQueue.remove();
+                        if (movingWindowSumHistogram instanceof DoubleHistogram) {
+                            if (prevHist != null) {
+                                ((DoubleHistogram) movingWindowSumHistogram).subtract((DoubleHistogram) prevHist);
+                            }
+                        } else {
+                            if (prevHist != null) {
+                                ((Histogram) movingWindowSumHistogram).subtract((Histogram) prevHist);
+                            }
+                        }
+                    }
+                    // Add interval histogram to moving window previous intervals memory:
+                    movingWindowQueue.add(intervalHistogram);
+                }
+
                 if ((firstStartTime == 0.0) && (logReader.getStartTimeSec() != 0.0)) {
                     firstStartTime = logReader.getStartTimeSec();
 
@@ -341,6 +403,36 @@ public class HistogramLogProcessor extends Thread {
                     }
                 }
 
+                if (movingWindowLog != null) {
+                    if (!movingWindowLogLegendWritten) {
+                        movingWindowLogLegendWritten = true;
+                        if (config.logFormatCsv) {
+                            movingWindowLog.println("\"Timestamp\",\"Window_Count\",\"" +
+                                    config.movingWindowPercentileToReport +"%'ile\",\"Max\"");
+                        } else {
+                            movingWindowLog.println("Time: WindoCount " + config.movingWindowPercentileToReport + "%'ile Max");
+                        }
+                    }
+                    if (intervalHistogram instanceof DoubleHistogram) {
+                        movingWindowLog.format(Locale.US, movingWindowLogFormat,
+                                ((intervalHistogram.getEndTimeStamp() / 1000.0) - logReader.getStartTimeSec()),
+                                // values recorded during the last reporting interval
+                                ((DoubleHistogram) movingWindowSumHistogram).getTotalCount(),
+                                ((DoubleHistogram) movingWindowSumHistogram).getValueAtPercentile(config.movingWindowPercentileToReport) / config.outputValueUnitRatio,
+                                ((DoubleHistogram) movingWindowSumHistogram).getMaxValue() / config.outputValueUnitRatio
+                                );
+                    } else {
+                        movingWindowLog.format(Locale.US, movingWindowLogFormat,
+                                ((intervalHistogram.getEndTimeStamp() / 1000.0) - logReader.getStartTimeSec()),
+                                // values recorded during the last reporting interval
+                                ((Histogram) movingWindowSumHistogram).getTotalCount(),
+                                ((Histogram) movingWindowSumHistogram).getValueAtPercentile(config.movingWindowPercentileToReport) / config.outputValueUnitRatio,
+                                ((Histogram) movingWindowSumHistogram).getMaxValue() / config.outputValueUnitRatio
+                                );
+                    }
+
+                }
+
                 intervalHistogram = getIntervalHistogram(config.tag);
             }
 
