@@ -17,6 +17,7 @@ import java.io.ByteArrayOutputStream;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 /**
  * JUnit test for {@link Histogram}
@@ -62,6 +63,161 @@ public class HistogramTest {
         Assert.assertEquals(histogram, null);
     }
 
+    @Test
+    public void testUnitMagnitude0IndexCalculations() {
+        Histogram h = new Histogram(1L, 1L << 32, 3);
+        assertEquals(2048, h.subBucketCount);
+        assertEquals(0, h.unitMagnitude);
+        // subBucketCount = 2^11, so 2^11 << 22 is > the max of 2^32 for 23 buckets total
+        assertEquals(23, h.bucketCount);
+
+        // first half of first bucket
+        assertEquals(0, h.getBucketIndex(3));
+        assertEquals(3, h.getSubBucketIndex(3, 0));
+
+        // second half of first bucket
+        assertEquals(0, h.getBucketIndex(1024 + 3));
+        assertEquals(1024 + 3, h.getSubBucketIndex(1024 + 3, 0));
+
+        // second bucket (top half)
+        assertEquals(1, h.getBucketIndex(2048 + 3 * 2));
+        // counting by 2s, starting at halfway through the bucket
+        assertEquals(1024 + 3, h.getSubBucketIndex(2048 + 3 * 2, 1));
+
+        // third bucket (top half)
+        assertEquals(2, h.getBucketIndex((2048 << 1) + 3 * 4));
+        // counting by 4s, starting at halfway through the bucket
+        assertEquals(1024 + 3, h.getSubBucketIndex((2048 << 1) + 3 * 4, 2));
+
+        // past last bucket -- not near Long.MAX_VALUE, so should still calculate ok.
+        assertEquals(23, h.getBucketIndex((2048L << 22) + 3 * (1 << 23)));
+        assertEquals(1024 + 3, h.getSubBucketIndex((2048L << 22) + 3 * (1 << 23), 23));
+    }
+
+    @Test
+    public void testUnitMagnitude4IndexCalculations() {
+        Histogram h = new Histogram(1L << 12, 1L << 32, 3);
+        assertEquals(2048, h.subBucketCount);
+        assertEquals(12, h.unitMagnitude);
+        // subBucketCount = 2^11. With unit magnitude shift, it's 2^23. 2^23 << 10 is > the max of 2^32 for 11 buckets
+        // total
+        assertEquals(11, h.bucketCount);
+        long unit = 1L << 12;
+
+        // below lowest value
+        assertEquals(0, h.getBucketIndex(3));
+        assertEquals(0, h.getSubBucketIndex(3, 0));
+
+        // first half of first bucket
+        assertEquals(0, h.getBucketIndex(3 * unit));
+        assertEquals(3, h.getSubBucketIndex(3 * unit, 0));
+
+        // second half of first bucket
+        // subBucketHalfCount's worth of units, plus 3 more
+        assertEquals(0, h.getBucketIndex(unit * (1024 + 3)));
+        assertEquals(1024 + 3, h.getSubBucketIndex(unit * (1024 + 3), 0));
+
+        // second bucket (top half), bucket scale = unit << 1.
+        // Middle of bucket is (subBucketHalfCount = 2^10) of bucket scale, = unit << 11.
+        // Add on 3 of bucket scale.
+        assertEquals(1, h.getBucketIndex((unit << 11) + 3 * (unit << 1)));
+        assertEquals(1024 + 3, h.getSubBucketIndex((unit << 11) + 3 * (unit << 1), 1));
+
+        // third bucket (top half), bucket scale = unit << 2.
+        // Middle of bucket is (subBucketHalfCount = 2^10) of bucket scale, = unit << 12.
+        // Add on 3 of bucket scale.
+        assertEquals(2, h.getBucketIndex((unit << 12) + 3 * (unit << 2)));
+        assertEquals(1024 + 3, h.getSubBucketIndex((unit << 12) + 3 * (unit << 2), 2));
+
+        // past last bucket -- not near Long.MAX_VALUE, so should still calculate ok.
+        assertEquals(11, h.getBucketIndex((unit << 21) + 3 * (unit << 11)));
+        assertEquals(1024 + 3, h.getSubBucketIndex((unit << 21) + 3 * (unit << 11), 11));
+    }
+
+    @Test
+    public void testUnitMagnitude51SubBucketMagnitude11IndexCalculations() {
+        // maximum unit magnitude for this precision
+        Histogram h = new Histogram(1L << 51, Long.MAX_VALUE, 3);
+        assertEquals(2048, h.subBucketCount);
+        assertEquals(51, h.unitMagnitude);
+        // subBucketCount = 2^11. With unit magnitude shift, it's 2^62. 1 more bucket to (almost) reach 2^63.
+        assertEquals(2, h.bucketCount);
+        assertEquals(2, h.leadingZeroCountBase);
+        long unit = 1L << 51;
+
+        // below lowest value
+        assertEquals(0, h.getBucketIndex(3));
+        assertEquals(0, h.getSubBucketIndex(3, 0));
+
+        // first half of first bucket
+        assertEquals(0, h.getBucketIndex(3 * unit));
+        assertEquals(3, h.getSubBucketIndex(3 * unit, 0));
+
+        // second half of first bucket
+        // subBucketHalfCount's worth of units, plus 3 more
+        assertEquals(0, h.getBucketIndex(unit * (1024 + 3)));
+        assertEquals(1024 + 3, h.getSubBucketIndex(unit * (1024 + 3), 0));
+
+        // end of second half
+        assertEquals(0, h.getBucketIndex(unit * 1024 + 1023 * unit));
+        assertEquals(1024 + 1023, h.getSubBucketIndex(unit * 1024 + 1023 * unit, 0));
+
+        // second bucket (top half), bucket scale = unit << 1.
+        // Middle of bucket is (subBucketHalfCount = 2^10) of bucket scale, = unit << 11.
+        // Add on 3 of bucket scale.
+        assertEquals(1, h.getBucketIndex((unit << 11) + 3 * (unit << 1)));
+        assertEquals(1024 + 3, h.getSubBucketIndex((unit << 11) + 3 * (unit << 1), 1));
+
+        // upper half of second bucket, last slot
+        assertEquals(1, h.getBucketIndex(Long.MAX_VALUE));
+        assertEquals(1024 + 1023, h.getSubBucketIndex(Long.MAX_VALUE, 1));
+    }
+
+    @Test
+    public void testUnitMagnitude52SubBucketMagnitude11Throws() {
+        try {
+            new Histogram(1L << 52, 1L << 62, 3);
+            fail();
+        } catch (IllegalArgumentException e) {
+            assertEquals("Cannot represent numberOfSignificantValueDigits worth of values beyond lowestDiscernibleValue",
+                    e.getMessage());
+        }
+    }
+
+    @Test
+    public void testUnitMagnitude54SubBucketMagnitude8Ok() {
+        Histogram h = new Histogram(1L << 54, 1L << 62, 2);
+        assertEquals(256, h.subBucketCount);
+        assertEquals(54, h.unitMagnitude);
+        // subBucketCount = 2^8. With unit magnitude shift, it's 2^62.
+        assertEquals(2, h.bucketCount);
+
+        // below lowest value
+        assertEquals(0, h.getBucketIndex(3));
+        assertEquals(0, h.getSubBucketIndex(3, 0));
+
+        // upper half of second bucket, last slot
+        assertEquals(1, h.getBucketIndex(Long.MAX_VALUE));
+        assertEquals(128 + 127, h.getSubBucketIndex(Long.MAX_VALUE, 1));
+    }
+
+    @Test
+    public void testUnitMagnitude61SubBucketMagnitude0Ok() {
+        Histogram h = new Histogram(1L << 61, 1L << 62, 0);
+        assertEquals(2, h.subBucketCount);
+        assertEquals(61, h.unitMagnitude);
+        // subBucketCount = 2^1. With unit magnitude shift, it's 2^62. 1 more bucket to be > the max of 2^62.
+        assertEquals(2, h.bucketCount);
+
+        // below lowest value
+        assertEquals(0, h.getBucketIndex(3));
+        assertEquals(0, h.getSubBucketIndex(3, 0));
+
+        // upper half of second bucket, last slot
+        assertEquals(1, h.getBucketIndex(Long.MAX_VALUE));
+        assertEquals(1, h.getSubBucketIndex(Long.MAX_VALUE, 1));
+    }
+
     @Test
     public void testEmptyHistogram() throws Exception {
         Histogram histogram = new Histogram(3);
@@ -141,11 +297,60 @@ public class HistogramTest {
         histogram.recordValue(20000000);
         histogram.recordValue(30000000);
         Assert.assertTrue(histogram.valuesAreEquivalent(20000000, histogram.getValueAtPercentile(50.0)));
-        Assert.assertTrue(histogram.valuesAreEquivalent(30000000, histogram.getValueAtPercentile(83.33)));
+        Assert.assertTrue(histogram.valuesAreEquivalent(30000000, histogram.getValueAtPercentile(50.0)));
+        Assert.assertTrue(histogram.valuesAreEquivalent(100000000, histogram.getValueAtPercentile(83.33)));
         Assert.assertTrue(histogram.valuesAreEquivalent(100000000, histogram.getValueAtPercentile(83.34)));
         Assert.assertTrue(histogram.valuesAreEquivalent(100000000, histogram.getValueAtPercentile(99.0)));
     }
 
+    @Test
+    public void testValueAtPercentileMatchesPercentile() throws Exception {
+        Histogram histogram = new Histogram(1, Long.MAX_VALUE, 3);
+        long[] lengths = {1, 5, 10, 50, 100, 500, 1000, 5000, 10000, 50000, 100000};
+
+
+        for (long length: lengths) {
+            histogram.reset();
+            for (long value = 1; value <= length; value++) {
+                histogram.recordValue(value);
+            }
+            
+            for (long value = 1; value <= length; value++) {
+                Double calculatedPercentile = 100.0 * ((double) value)/length;
+                long lookupValue = histogram.getValueAtPercentile(calculatedPercentile);
+                Assert.assertTrue("length:" + length + " value: " + value + " calculatedPercentile:" + calculatedPercentile +
+                        " getValueAtPercentile(" + calculatedPercentile + ") = " + lookupValue +
+                        " [should be " + value + "]",
+                        histogram.valuesAreEquivalent(value, lookupValue));
+            }
+        }
+    }
+
+    @Test
+    public void testValueAtPercentileMatchesPercentileIter() throws Exception {
+        Histogram histogram = new Histogram(1, Long.MAX_VALUE, 3);
+        long[] lengths = {1, 5, 10, 50, 100, 500, 1000, 5000, 10000, 50000, 100000};
+
+
+        for (long length: lengths) {
+            histogram.reset();
+            for (long value = 1; value <= length; value++) {
+                histogram.recordValue(value);
+            }
+
+            int percentileTicksPerHalfDistance = 1000;
+            for (HistogramIterationValue v : histogram.percentiles(percentileTicksPerHalfDistance)) {
+                long calculatedValue = histogram.getValueAtPercentile(v.getPercentile());
+                long iterValue = v.getValueIteratedTo();
+                Assert.assertTrue("length:" + length + " percentile: " + v.getPercentile() +
+                                " calculatedValue:" + calculatedValue + " iterValue:" + iterValue +
+                                "[should be " + calculatedValue + "]",
+                        histogram.valuesAreEquivalent(calculatedValue, iterValue));
+                Assert.assertTrue(histogram.valuesAreEquivalent(calculatedValue, iterValue));
+            }
+        }
+    }
+
     @Test
     public void testRecordValueWithExpectedInterval() throws Exception {
         Histogram histogram = new Histogram(highestTrackableValue, numberOfSignificantValueDigits);
@@ -219,7 +424,7 @@ public class HistogramTest {
     }
 
     @Test
-    public void testSubtract() throws Exception {
+    public void testSubtractAfterAdd() {
         Histogram histogram = new Histogram(highestTrackableValue, numberOfSignificantValueDigits);
         Histogram other = new Histogram(highestTrackableValue, numberOfSignificantValueDigits);
         histogram.recordValue(testValueLevel);
@@ -238,21 +443,76 @@ public class HistogramTest {
         Assert.assertEquals(2L, histogram.getCountAtValue(testValueLevel));
         Assert.assertEquals(2L, histogram.getCountAtValue(testValueLevel * 1000));
         Assert.assertEquals(4L, histogram.getTotalCount());
+
+        verifyMaxValue(histogram);
+        verifyMaxValue(other);
+    }
+
+    @Test
+    public void testSubtractToZeroCounts() {
+        Histogram histogram = new Histogram(highestTrackableValue, numberOfSignificantValueDigits);
+        histogram.recordValue(testValueLevel);
+        histogram.recordValue(testValueLevel * 1000);
+        Assert.assertEquals(1L, histogram.getCountAtValue(testValueLevel));
+        Assert.assertEquals(1L, histogram.getCountAtValue(testValueLevel * 1000));
+        Assert.assertEquals(2L, histogram.getTotalCount());
+
         // Subtracting down to zero counts should work:
         histogram.subtract(histogram);
         Assert.assertEquals(0L, histogram.getCountAtValue(testValueLevel));
         Assert.assertEquals(0L, histogram.getCountAtValue(testValueLevel * 1000));
         Assert.assertEquals(0L, histogram.getTotalCount());
-        // But subtracting down to negative counts should not:
-        boolean thrown = false;
-        try {
-            // This should throw:
-            histogram.subtract(other);
-        } catch (IllegalArgumentException e) {
-            thrown = true;
-        }
-        Assert.assertTrue(thrown);
 
+        verifyMaxValue(histogram);
+    }
+
+    @Test
+    public void testSubtractToNegativeCountsThrows() {
+        Histogram histogram = new Histogram(highestTrackableValue, numberOfSignificantValueDigits);
+        Histogram other = new Histogram(highestTrackableValue, numberOfSignificantValueDigits);
+        histogram.recordValue(testValueLevel);
+        histogram.recordValue(testValueLevel * 1000);
+        other.recordValueWithCount(testValueLevel, 2);
+        other.recordValueWithCount(testValueLevel * 1000, 2);
+
+        try {
+            histogram.subtract(other);
+            fail();
+        } catch (IllegalArgumentException e) {
+            // should throw
+        }
+
+        verifyMaxValue(histogram);
+        verifyMaxValue(other);
+    }
+
+    @Test
+    public void testSubtractSubtrahendValuesOutsideMinuendRangeThrows() {
+        Histogram histogram = new Histogram(highestTrackableValue, numberOfSignificantValueDigits);
+        histogram.recordValue(testValueLevel);
+        histogram.recordValue(testValueLevel * 1000);
+
+        Histogram biggerOther = new Histogram(highestTrackableValue * 2, numberOfSignificantValueDigits);
+        biggerOther.recordValue(testValueLevel);
+        biggerOther.recordValue(testValueLevel * 1000);
+        biggerOther.recordValue(highestTrackableValue * 2); // outside smaller histogram's range
+
+        try {
+            histogram.subtract(biggerOther);
+            fail();
+        } catch (IllegalArgumentException e) {
+            // should throw
+        }
+
+        verifyMaxValue(histogram);
+        verifyMaxValue(biggerOther);
+    }
+
+    @Test
+    public void testSubtractSubtrahendValuesInsideMinuendRangeWorks() {
+        Histogram histogram = new Histogram(highestTrackableValue, numberOfSignificantValueDigits);
+        histogram.recordValue(testValueLevel);
+        histogram.recordValue(testValueLevel * 1000);
 
         Histogram biggerOther = new Histogram(highestTrackableValue * 2, numberOfSignificantValueDigits);
         biggerOther.recordValue(testValueLevel);
@@ -266,33 +526,27 @@ public class HistogramTest {
         Assert.assertEquals(12L, biggerOther.getTotalCount());
 
         // Subtracting the smaller histogram from the bigger one should work:
-        biggerOther.subtract(other);
+        biggerOther.subtract(histogram);
         Assert.assertEquals(3L, biggerOther.getCountAtValue(testValueLevel));
         Assert.assertEquals(3L, biggerOther.getCountAtValue(testValueLevel * 1000));
         Assert.assertEquals(4L, biggerOther.getCountAtValue(highestTrackableValue * 2)); // overflow smaller hist...
         Assert.assertEquals(10L, biggerOther.getTotalCount());
 
-        // But trying to subtract a larger histogram into a smaller one should throw an AIOOB:
-        thrown = false;
-        try {
-            // This should throw:
-            histogram.subtract(biggerOther);
-        } catch (ArrayIndexOutOfBoundsException e) {
-            thrown = true;
-        }
-        Assert.assertTrue(thrown);
-
         verifyMaxValue(histogram);
-        verifyMaxValue(other);
         verifyMaxValue(biggerOther);
     }
 
-
     @Test
     public void testSizeOfEquivalentValueRange() {
         Histogram histogram = new Histogram(highestTrackableValue, numberOfSignificantValueDigits);
         Assert.assertEquals("Size of equivalent range for value 1 is 1",
                 1, histogram.sizeOfEquivalentValueRange(1));
+        Assert.assertEquals("Size of equivalent range for value 1025 is 1",
+                1, histogram.sizeOfEquivalentValueRange(1025));
+        Assert.assertEquals("Size of equivalent range for value 2047 is 1",
+                1, histogram.sizeOfEquivalentValueRange(2047));
+        Assert.assertEquals("Size of equivalent range for value 2048 is 2",
+                2, histogram.sizeOfEquivalentValueRange(2048));
         Assert.assertEquals("Size of equivalent range for value 2500 is 2",
                 2, histogram.sizeOfEquivalentValueRange(2500));
         Assert.assertEquals("Size of equivalent range for value 8191 is 4",
@@ -420,6 +674,10 @@ public class HistogramTest {
         histogram.recordValue(testValueLevel);
         histogram.recordValue(testValueLevel * 10);
         histogram.recordValueWithExpectedInterval(histogram.getHighestTrackableValue() - 1, 255);
+        if (histogram.supportsAutoResize()) {
+            histogram.setAutoResize(true);
+            assertTrue(histogram.isAutoResize());
+        }
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         ObjectOutput out = null;
         ByteArrayInputStream bis = null;
@@ -450,6 +708,10 @@ public class HistogramTest {
         Assert.assertNotNull(newHistogram);
         assertEqual(histogram, newHistogram);
         assertTrue(histogram.equals(newHistogram));
+        if (histogram.supportsAutoResize()) {
+            assertTrue(histogram.isAutoResize());
+        }
+        assertEquals(newHistogram.isAutoResize(), histogram.isAutoResize());
         Assert.assertTrue(histogram.hashCode() == newHistogram.hashCode());
         assertEquals(histogram.getNeededByteBufferCapacity(), newHistogram.copy().getNeededByteBufferCapacity());
         assertEquals(histogram.getNeededByteBufferCapacity(), newHistogram.getNeededByteBufferCapacity());
